"""
A set of convenient utilities for numerical work.

Most of this module requires Numerical Python or is meant to be used with it.
See http://www.pfdubois.com/numpy for details.
"""

#*****************************************************************************
#       Copyright (C) 2001 Fernando Prez. <fperez@pizero.colorado.edu>
#
#  Distributed under the terms of the GNU Lesser General Public License (LGPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#  The full text of the LGPL is available at:
#
#                  http://www.gnu.org/copyleft/lesser.html
#*****************************************************************************

__author__ = 'Fernando Prez. <fperez@pizero.colorado.edu>'
__version__= '0.1.1'
__license__ = 'LGPL'
__date__   = 'Tue Dec 11 00:27:58 MST 2001'

from genutils import qw
__all__ = qw('gnuplot_exec count_elements sum_flat mean_flat '
             'rms_flat frange diagonal_matrix fromfunction inf infty Infinity')

#****************************************************************************
# required modules
from Numeric import *
import sys, ultraTB
import __main__

AutoTB = ultraTB.AutoFormattedTB(mode='Verbose',color_scheme='Linux')

#*****************************************************************************
# Globals

# useful for testing infinities in results of array divisions (which don't
# raise an exception)
# Python, LaTeX and Mathematica names.
inf = infty = Infinity = (array([1])/0.0)[0]


#****************************************************************************
# function definitions

def gnuplot_exec(gnuplot,string_of_commands,verbose=0):
    """Have gnuplot execute a string of commands, one per line."""

    if verbose:
        print 'Calling gnuplot to execute:'
    for cmd in string_of_commands.splitlines():
        if verbose:
            print cmd
        gnuplot(cmd)


#-----------------------------------------------------------------------------
def count_elements(a):
    """Return the total number of elements of the input (may be
    multi-dimensional) array.

    It is simply a call to product(a.shape)"""

    return product(a.shape)

#-----------------------------------------------------------------------------
def sum_flat(a):
    """Return the sum of all the elements of a, flattened out (recursively)."""

    if type(a) is not ArrayType: return a
    return sum_flat(sum(a))

#-----------------------------------------------------------------------------
def mean_flat(a):
    """Return the mean of all the elements of a, flattened out (recursively)."""

    return sum_flat(a)/float(product(a.shape))

#-----------------------------------------------------------------------------
def rms_flat(a):
    """Return the root mean square of all the elements of a, flattened out
    (recursively)."""

    return sqrt(sum_flat(a**2)/float(product(a.shape)))

#----------------------------------------------------------------------------
def frange(xini,xfin=None,delta=None,**kw):
    """frange([start,] stop[, step, keywords]) -> list of floats

    Return a Numeric array() containing a progression of floats. Similar to
    arange(), but defaults to a closed interval.

    frange(x0, x1) returns [x0, x0+1, x0+2, ..., x1]; start defaults to 0, and
    the endpoint *is included*. This behavior is different from that of
    range() and arange(). This is deliberate, since frange will probably be
    more useful for generating lists of points for function evaluation, and
    endpoints are often desired in this use. The usual behavior of range() can
    be obtained by setting the keyword 'closed=0', in this case frange()
    basically becomes arange().

    When step is given, it specifies the increment (or decrement). All
    arguments can be floating point numbers.

    frange(x0,x1,d) returns [x0,x0+d,x0+2d,...,xfin] where xfin<=x1.

    frange can also be called with the keyword 'npts'. This sets the number of
    points the list should contain (and overrides the value 'step' might have
    been given). arange() doesn't offer this option.

    Examples:
    >>> frange(3)
    array([ 0.,  1.,  2.,  3.])
    >>> frange(3,closed=0)
    array([ 0.,  1.,  2.])
    >>> frange(1,6,2)
    array([1, 3, 5])
    >>> frange(1,6.5,npts=5)
    array([ 1.   ,  2.375,  3.75 ,  5.125,  6.5  ])
    """

    #defaults
    kw.setdefault('closed',1)
    endpoint = kw['closed'] != 0
        
    # funny logic to allow the *first* argument to be optional (like range())
    # This was modified with a simpler version from a similar frange() found
    # at http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66472
    if xfin == None:
        xfin = xini + 0.0
        xini = 0.0
        
    if delta == None:
        delta = 1.0

    # compute # of points, spacing and return final list
    try:
        npts=kw['npts']
        delta=(xfin-xini)/float(npts-endpoint)
    except KeyError:
        npts=int((xfin-xini)/delta+endpoint)

    return arange(npts)*delta+xini
# end of frange()

#-----------------------------------------------------------------------------
def diagonal_matrix(diag):
    """Return square diagonal matrix whose non-zero elements are given by the
    input array."""

    return diag*identity(len(diag))

#-----------------------------------------------------------------------------
def fromfunction_kw(function, dimensions, **kwargs):
    """Drop-in replacement for fromfunction() from Numerical Python.
 
    Allows passing keyword arguments to the desired function. Since it doesn't
    remove any of Numerical fromfunction's functionality, it's ok to use the
    same name and overwrite the original.

    Call it as (keywords are optional):
    fromfunction_kw(MyFunction, dimensions, keywords)

    The function MyFunction() is responsible for handling the dictionary of
    keywords it will recieve."""

    return function(tuple(indices(dimensions)),**kwargs)

#**************************** end of file <numutils.py> ************************
